@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: CL-PPCRE; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/cl-ppcre/api.lisp,v 1.48 2004/12/06 11:34:15 edi Exp $
+;;; $Header: /usr/local/cvsrep/cl-ppcre/api.lisp,v 1.54 2005/03/07 22:02:52 edi Exp $
 
 ;;; The external API for creating and using scanners.
 
@@ -43,6 +43,7 @@ are equivalent to the imsx modifiers in Perl. If DESTRUCTIVE is not
 NIL the function is allowed to destructively modify its first argument
 \(but only if it's a parse tree)."))
 
+#-:use-acl-regexp2-engine
 (defmethod create-scanner ((regex-string string) &key case-insensitive-mode
                                                       multi-line-mode
                                                       single-line-mode
@@ -51,7 +52,7 @@ NIL the function is allowed to destructively modify its first argument
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (declare (ignore destructive))
@@ -70,6 +71,7 @@ NIL the function is allowed to destructively modify its first argument
                     :single-line-mode single-line-mode
                     :destructive t)))
 
+#-:use-acl-regexp2-engine
 (defmethod create-scanner ((scanner function) &key case-insensitive-mode
                                                    multi-line-mode
                                                    single-line-mode
@@ -78,7 +80,7 @@ NIL the function is allowed to destructively modify its first argument
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (declare (ignore destructive))
@@ -87,6 +89,7 @@ NIL the function is allowed to destructively modify its first argument
      "You can't use the keyword arguments to modify an existing scanner."))
   scanner)
 
+#-:use-acl-regexp2-engine
 (defmethod create-scanner ((parse-tree t) &key case-insensitive-mode
                                                multi-line-mode
                                                single-line-mode
@@ -95,7 +98,7 @@ NIL the function is allowed to destructively modify its first argument
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (when extended-mode
@@ -188,6 +191,35 @@ NIL the function is allowed to destructively modify its first argument
                               *zero-length-num*
                               reg-num))))))
 
+#+:use-acl-regexp2-engine
+(declaim (inline create-scanner))
+
+#+:use-acl-regexp2-engine
+(defmethod create-scanner ((scanner regexp::regular-expression) &key case-insensitive-mode
+                                                                     multi-line-mode
+                                                                     single-line-mode
+                                                                     extended-mode
+                                                                     destructive)
+  (declare (ignore destructive))
+  (when (or case-insensitive-mode multi-line-mode single-line-mode extended-mode)
+    (signal-ppcre-invocation-error
+     "You can't use the keyword arguments to modify an existing scanner."))
+  scanner)
+
+#+:use-acl-regexp2-engine
+(defmethod create-scanner ((parse-tree t) &key case-insensitive-mode
+                                               multi-line-mode
+                                               single-line-mode
+                                               extended-mode
+                                               destructive)
+  (declare (ignore destructive))
+  (excl:compile-re parse-tree
+                   :case-fold case-insensitive-mode
+                   :ignore-whitespace extended-mode
+                   :multiple-lines multi-line-mode
+                   :single-line single-line-mode
+                   :return :index))
+
 (defgeneric scan (regex target-string &key start end)
   (:documentation "Searches TARGET-STRING from START to END and tries
 to match REGEX. On success returns four values - the start of the
@@ -197,13 +229,14 @@ string which will be parsed according to Perl syntax, a parse tree, or
 a pre-compiled scanner created by CREATE-SCANNER. TARGET-STRING will
 be coerced to a simple string if it isn't one already."))
 
+#-:use-acl-regexp2-engine
 (defmethod scan ((regex-string string) target-string
                                        &key (start 0)
                                             (end (length target-string)))
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   ;; note that the scanners are optimized for simple strings so we
@@ -212,32 +245,61 @@ be coerced to a simple string if it isn't one already."))
            (maybe-coerce-to-simple-string target-string)
            start end))
 
+#-:use-acl-regexp2-engine
 (defmethod scan ((scanner function) target-string
                                     &key (start 0)
                                          (end (length target-string)))
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (funcall scanner
            (maybe-coerce-to-simple-string target-string)
            start end))
 
+#-:use-acl-regexp2-engine
 (defmethod scan ((parse-tree t) target-string
                                 &key (start 0)
                                      (end (length target-string)))
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (funcall (create-scanner parse-tree)
            (maybe-coerce-to-simple-string target-string)
            start end))
 
+#+:use-acl-regexp2-engine
+(declaim (inline scan))
+
+#+:use-acl-regexp2-engine
+(defmethod scan ((parse-tree t) target-string
+                                &key (start 0)
+                                     (end (length target-string)))
+  (when (< end start)
+    (return-from scan nil))
+  (let ((results (multiple-value-list (excl:match-re parse-tree target-string
+                                                     :start start
+                                                     :end end
+                                                     :return :index))))
+    (declare (dynamic-extent results))
+    (cond ((null (first results)) nil)
+          (t (let* ((no-of-regs (- (length results) 2))
+                    (reg-starts (make-array no-of-regs
+                                            :element-type '(or null fixnum)))
+                    (reg-ends (make-array no-of-regs
+                                          :element-type '(or null fixnum)))
+                    (match (second results)))
+               (loop for (start . end) in (cddr results)
+                     for i from 0
+                     do (setf (aref reg-starts i) start
+                              (aref reg-ends i) end))
+               (values (car match) (cdr match) reg-starts reg-ends))))))
+
 (define-compiler-macro scan (&whole form &environment env regex target-string &rest rest)
   "Make sure that constant forms are compiled into scanners at compile time."
   (cond ((constantp regex env)
@@ -252,7 +314,7 @@ be coerced to a simple string if it isn't one already."))
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Like SCAN but returns substrings of TARGET-STRING instead of
@@ -296,7 +358,7 @@ VAR-LIST which is NIL there's no binding to the corresponding register
 group. The number of variables in VAR-LIST must not be greater than
 the number of register groups. If SHAREDP is true, the substrings may
 share structure with TARGET-STRING."
-  (rebinding (target-string)
+  (with-rebinding (target-string)
     (with-unique-names (match-start match-end reg-starts reg-ends
                                     start-index substr-fn)
       `(multiple-value-bind (,match-start ,match-end ,reg-starts ,reg-ends)
@@ -326,7 +388,8 @@ share structure with TARGET-STRING."
                                  target-string
                                  &optional result-form
                                  &key start end)
-                    &body body)
+                    &body body
+                    &environment env)
   "Iterates over TARGET-STRING and tries to match REGEX as often as
 possible evaluating BODY with MATCH-START, MATCH-END, REG-STARTS, and
 REG-ENDS bound to the four return values of each match in turn. After
@@ -335,19 +398,24 @@ implicit block named NIL surrounds DO-SCANS; RETURN may be used to
 terminate the loop immediately. If REGEX matches an empty string the
 scan is continued one position behind this match. BODY may start with
 declarations."
-  (rebinding (target-string regex)
-    (with-unique-names (%start %end scanner loop-tag block-name)
+  (with-rebinding (target-string)
+    (with-unique-names (%start %end %regex scanner loop-tag block-name)
+      (declare (ignorable %regex scanner))
       ;; the NIL BLOCK to enable exits via (RETURN ...)
       `(block nil
         (let* ((,%start (or ,start 0))
                (*real-start-pos* ,%start)
                (,%end (or ,end (length ,target-string)))
-               ;; create a scanner unless the regex is already a
-               ;; function (otherwise SCAN will do this on each
-               ;; iteration)
-               (,scanner (typecase ,regex
-                           (function ,regex)
-                           (t (create-scanner ,regex)))))
+               ,@(unless (constantp regex env)
+                   ;; leave constant regular expressions as they are -
+                   ;; SCAN's compiler macro will take care of them;
+                   ;; otherwise create a scanner unless the regex is
+                   ;; already a function (otherwise SCAN will do this
+                   ;; on each iteration)
+                   `((,%regex ,regex)
+                     (,scanner (typecase ,%regex
+                                 (function ,%regex)
+                                 (t (create-scanner ,%regex)))))))
           ;; coerce TARGET-STRING to a simple string unless it is one
           ;; already (otherwise SCAN will do this on each iteration)
           (setq ,target-string
@@ -360,7 +428,9 @@ declarations."
               ;; provided variables
               (multiple-value-bind
                     (,match-start ,match-end ,reg-starts ,reg-ends)
-                  (scan ,scanner ,target-string :start ,%start :end ,%end)
+                  (scan ,(cond ((constantp regex env) regex)
+                               (t scanner))
+                        ,target-string :start ,%start :end ,%end)
                 ;; declare the variables to be IGNORABLE to prevent the
                 ;; compiler from issuing warnings
                 (declare
@@ -415,7 +485,7 @@ terminate the loop immediately. If REGEX matches an empty string the
 scan is continued one position behind this match. If SHAREDP is true,
 the substrings may share structure with TARGET-STRING. BODY may start
 with declarations."
-  (rebinding (target-string)
+  (with-rebinding (target-string)
     (with-unique-names (match-start match-end substr-fn)
       `(let ((,substr-fn (if ,sharedp #'nsubseq #'subseq)))
         ;; simple use DO-MATCHES to extract the substrings
@@ -442,7 +512,7 @@ surrounds DO-REGISTER-GROUPS; RETURN may be used to terminate the loop
 immediately. If REGEX matches an empty string the scan is continued
 one position behind this match. If SHAREDP is true, the substrings may
 share structure with TARGET-STRING. BODY may start with declarations."
-  (rebinding (target-string)
+  (with-rebinding (target-string)
     (with-unique-names (substr-fn match-start match-end
                                   reg-starts reg-ends start-index)
       `(let ((,substr-fn (if ,sharedp
@@ -471,7 +541,7 @@ share structure with TARGET-STRING. BODY may start with declarations."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Returns a list containing the start and end positions of all
@@ -502,7 +572,7 @@ compile time."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Returns a list containing all substrings of TARGET-STRING which
@@ -534,7 +604,7 @@ compile time."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Matches REGEX against TARGET-STRING as often as possible and
@@ -612,7 +682,7 @@ TARGET-STRING."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (declare (type fixnum from to start end))
@@ -681,12 +751,12 @@ that (<= START FROM TO END)."
 ;; replacement string (eat your own dog food...)
 #-:cormanlisp
 (let* ((*use-bmh-matchers* nil)
-       (reg-scanner (create-scanner "\\\\(?:\\\\|{\\d+}|\\d+|&|`|')")))
+       (reg-scanner (create-scanner "\\\\(?:\\\\|\\{\\d+\\}|\\d+|&|`|')")))
   (defmethod build-replacement-template ((replacement-string string))
     (declare (optimize speed
                        (safety 0)
                        (space 0)
-                       (debug 0)
+                       (debug 1)
                        (compilation-speed 0)
                        #+:lispworks (hcl:fixnum-safety 0)))
     "Converts a replacement string for REGEX-REPLACE or
@@ -745,12 +815,12 @@ S-expression."
 ;;; Corman Lisp's methods can't be closures... :(
 #+:cormanlisp
 (let* ((*use-bmh-matchers* nil)
-       (reg-scanner (create-scanner "\\\\(?:\\\\|{\\d+}|\\d+|&|`|')")))
+       (reg-scanner (create-scanner "\\\\(?:\\\\|\\{\\d+\\}|\\d+|&|`|')")))
   (defun build-replacement-template (replacement)
     (declare (optimize speed
                        (safety 0)
                        (space 0)
-                       (debug 0)
+                       (debug 1)
                        (compilation-speed 0)
                        #+:lispworks (hcl:fixnum-safety 0)))
     (typecase replacement
@@ -806,7 +876,7 @@ S-expression."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Accepts a replacement template and the current values from the
@@ -897,7 +967,7 @@ corresponding template."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Auxiliary function used by REGEX-REPLACE and
@@ -946,7 +1016,7 @@ representing the corresponding register start and end positions."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Try to match TARGET-STRING between START and END against REGEX and
@@ -1001,7 +1071,7 @@ match."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Try to match TARGET-STRING between START and END against REGEX and
@@ -1063,7 +1133,7 @@ through PACKAGES and executes BODY with SYMBOL bound to each symbol
 which matches REGEX. Optionally evaluates and returns RETURN-FORM at
 the end. If CASE-INSENSITIVE is true and REGEX isn't already a
 scanner, a case-insensitive scanner is used."
-  (rebinding (regex)
+  (with-rebinding (regex)
     (with-unique-names (scanner %packages next morep)
       `(let* ((,scanner (create-scanner ,regex
                                         :case-insensitive-mode
@@ -1100,7 +1170,7 @@ through PACKAGES and executes BODY with SYMBOL bound to each symbol
 which matches REGEX. Optionally evaluates and returns RETURN-FORM at
 the end. If CASE-INSENSITIVE is true and REGEX isn't already a
 scanner, a case-insensitive scanner is used."
-  (rebinding (regex)
+  (with-rebinding (regex)
     (with-unique-names (scanner %packages)
       `(let* ((,scanner (create-scanner ,regex
                          :case-insensitive-mode
@@ -1117,7 +1187,7 @@ scanner, a case-insensitive scanner is used."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Similar to the standard function APROPOS-LIST but returns a list of
@@ -1134,7 +1204,7 @@ meaningful information about a symbol."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (handler-case
@@ -1184,7 +1254,7 @@ is used."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (regex-apropos-aux (regex packages case-insensitive)
