@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: CL-PPCRE; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/cl-ppcre/convert.lisp,v 1.15 2004/05/19 05:16:49 edi Exp $
+;;; $Header: /usr/local/cvsrep/cl-ppcre/convert.lisp,v 1.20 2005/03/07 22:02:53 edi Exp $
 
 ;;; Here the parse tree is converted into its internal representation
 ;;; using REGEX objects.  At the same time some optimizations are
@@ -53,7 +53,7 @@
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (declare (special flags))
@@ -79,7 +79,7 @@ the special FLAGS list."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (declare (special flags))
@@ -105,7 +105,7 @@ case-(in)sensitivity as specified by the special variable FLAGS."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Combines all items in LIST into one char class hash and returns it.
@@ -115,7 +115,7 @@ thing with respect to case-\(in)sensitivity as specified by the
 special variable FLAGS."
   (loop with hash = (make-hash-table :size (ceiling (expt *regex-char-code-limit* (/ 1 4)))
                                      :rehash-size (float (expt *regex-char-code-limit* (/ 1 4)))
-                                     :rehash-threshold 1.0)
+                                     :rehash-threshold #-genera 1.0 #+genera 0.99)
         for item in list
         if (characterp item)
           ;; treat a single character C like a range (:RANGE C C)
@@ -160,7 +160,7 @@ special variable FLAGS."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (declare (type fixnum minimum)
@@ -230,11 +230,11 @@ the same name."
 ;; case if the regex starts with ".*" which implicitely anchors the
 ;; regex at the start (perhaps modulo #\Newline).
 
-(defmethod maybe-accumulate ((str str))
+(defun maybe-accumulate (str)
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (declare (special accumulate-start-p starts-with))
@@ -294,7 +294,7 @@ NIL or a STR object of the same case mode. Always returns NIL."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (declare (special flags reg-num accumulate-start-p starts-with max-back-ref))
@@ -538,8 +538,17 @@ Will also
                 (make-instance 'register
                                :regex (convert-aux (second parse-tree))
                                :num stored-reg-num)))
+            ;; (:FILTER <function> &optional <length>)
+            ((:filter)
+              ;; stop accumulating into STARTS-WITH
+              (setq accumulate-start-p nil)
+              (make-instance 'filter
+                             :fn (second parse-tree)
+                             :len (third parse-tree)))
             ;; (:STANDALONE <regex>)
             ((:standalone)
+              ;; stop accumulating into STARTS-WITH
+              (setq accumulate-start-p nil)
               ;; keep the effect of modifiers local to the enclosed
               ;; regex
               (let ((flags (copy-list flags)))
@@ -702,7 +711,6 @@ Will also
                                         ; Perl's "\z"
               :modeless-start-anchor    ; Perl's "\A"
               :modeless-end-anchor)     ; Perl's "\Z"
-;              (setq accumulate-start-p nil)
               (make-instance 'anchor
                              :startp (member parse-tree
                                              '(:start-anchor
@@ -740,14 +748,18 @@ Will also
               (set-flag parse-tree)
               (make-instance 'void))
             (otherwise
-              (signal-ppcre-syntax-error "Unknown token ~A in parse-tree"
-                                         parse-tree))))))
+             (let ((translation (and (symbolp parse-tree)
+                                     (parse-tree-synonym parse-tree))))
+               (if translation
+                 (convert-aux (copy-tree translation))
+                 (signal-ppcre-syntax-error "Unknown token ~A in parse-tree"
+                                            parse-tree))))))))
 
 (defun convert (parse-tree)
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Converts the parse tree PARSE-TREE into an equivalent REGEX object
