@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: CL-PPCRE; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/cl-ppcre/lexer.lisp,v 1.20 2004/04/20 11:37:35 edi Exp $
+;;; $Header: /usr/local/cvsrep/cl-ppcre/lexer.lisp,v 1.22 2005/03/07 22:02:53 edi Exp $
 
 ;;; The lexer's responsibility is to convert the regex string into a
 ;;; sequence of tokens which are in turn consumed by the parser.
@@ -42,7 +42,7 @@
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Maps escaped characters like \"\\d\" to the tokens which represent
@@ -65,7 +65,7 @@ their associated character classes."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (defstruct (lexer (:constructor make-lexer-internal))
@@ -86,7 +86,7 @@ currently lexed and to keep track of the lexer's state."
 
 (defun make-lexer (string)
   (declare (inline make-lexer-internal)
-           (type string string))
+           #-genera (type string string))
   (make-lexer-internal :str (maybe-coerce-to-simple-string string)
                        :len (length string)))
 
@@ -95,7 +95,7 @@ currently lexed and to keep track of the lexer's state."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Tests whether we're at the end of the regex string."
@@ -107,7 +107,7 @@ currently lexed and to keep track of the lexer's state."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Tests whether the next character the lexer would see is CHR.
@@ -121,7 +121,7 @@ Does not respect extended mode."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Returns the next character which is to be examined and updates the
@@ -137,7 +137,7 @@ POS slot. Does not respect extended mode."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Returns the next character which is to be examined and updates the
@@ -206,7 +206,7 @@ nested comments are skipped if applicable."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Moves (LEXER-POS LEXER) back to the last position stored in
@@ -220,7 +220,7 @@ nested comments are skipped if applicable."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Read and consume the number the lexer is currently looking at and
@@ -255,7 +255,7 @@ we don't tolerate whitespace in front of the number."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Like GET-NUMBER but won't consume anything if no number is seen."
@@ -272,7 +272,7 @@ we don't tolerate whitespace in front of the number."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Create character from char-code NUMBER. NUMBER can be NIL
@@ -291,7 +291,7 @@ the corresponding number started within the regex string."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Convert the characters(s) following a backslash into a token
@@ -354,7 +354,7 @@ handled elsewhere."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Reads and consumes characters from regex string until a right
@@ -440,7 +440,7 @@ we're inside a range or not."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Reads a sequence of modifiers \(including #\\- to reverse their
@@ -481,7 +481,7 @@ the behaviour of the lexer itself via the special variable
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Returns a list of two values (min max) if what the lexer is looking
@@ -536,7 +536,7 @@ resets the lexer to its old position."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Returns and consumes the next token from the regex string (or NIL)."
@@ -740,7 +740,7 @@ resets the lexer to its old position."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Moves the lexer back to the last position stored in the LAST-POS stack."
@@ -754,7 +754,7 @@ resets the lexer to its old position."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Tests whether the next token can start a valid sub-expression, i.e.
@@ -766,4 +766,4 @@ a stand-alone regex."
                (member (the character next-char)
                        '(#\) #\|)
                        :test #'char=)
-               (setf (lexer-pos lexer) pos))))))
\ No newline at end of file
+               (setf (lexer-pos lexer) pos))))))
