@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: CL-PPCRE; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/cl-ppcre/optimize.lisp,v 1.21 2004/10/14 12:40:39 edi Exp $
+;;; $Header: /usr/local/cvsrep/cl-ppcre/optimize.lisp,v 1.23 2005/03/07 22:02:53 edi Exp $
 
 ;;; This file contains optimizations which can be applied to converted
 ;;; parse trees.
@@ -32,35 +32,11 @@
 
 (in-package #:cl-ppcre)
 
-(defun string-list-to-simple-string (string-list)
-  (declare (optimize speed
-                     (safety 0)
-                     (space 0)
-                     (debug 0)
-                     (compilation-speed 0)
-                     #+:lispworks (hcl:fixnum-safety 0)))
-  "Concatenates a list of strings to one simple-string."
-  ;; this function provided by JP Massar; note that we can't use APPLY
-  ;; with CONCATENATE here because of CALL-ARGUMENTS-LIMIT
-  (let ((total-size 0))
-    (declare (type fixnum total-size))
-    (dolist (string string-list)
-      #-genera (declare (type string string))
-      (incf total-size (length string)))
-    (let ((result-string (make-sequence 'simple-string total-size))
-          (curr-pos 0))
-      (declare (type fixnum curr-pos))
-      (dolist (string string-list)
-        #-genera (declare (type string string))
-        (replace result-string string :start1 curr-pos)
-        (incf curr-pos (length string)))
-      result-string)))
-
 (defgeneric flatten (regex)
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (:documentation "Merges adjacent sequences and alternations, i.e. it
@@ -156,7 +132,7 @@ operation on REGEX."))
     (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (:documentation "Collects adjacent strings or characters into one
@@ -320,7 +296,7 @@ operation on REGEX."))
     (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (:documentation "Returns T if REGEX starts with a \"real\" start
@@ -394,7 +370,7 @@ zero-length assertion."))
     (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (:documentation "Returns the constant string (if it exists) REGEX
@@ -523,7 +499,7 @@ function called by END-STRIN.)"))
     (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Returns the constant string (if it exists) REGEX ends with wrapped
@@ -548,7 +524,7 @@ into a STR object, otherwise NIL."
     (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (:documentation "Returns the minimal length of REGEX plus
