@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: CL-PPCRE-TEST; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/cl-ppcre/ppcre-tests.lisp,v 1.25 2004/04/20 11:37:36 edi Exp $
+;;; $Header: /usr/local/cvsrep/cl-ppcre/ppcre-tests.lisp,v 1.27 2005/03/07 22:02:53 edi Exp $
 
 ;;; Copyright (c) 2002-2004, Dr. Edmund Weitz. All rights reserved.
 
@@ -27,18 +27,6 @@
 ;;; NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;; SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-(in-package #:cl-user)
-
-#-:cormanlisp
-(defpackage #:cl-ppcre-test
-  (:use #:cl #:cl-ppcre)
-  (:export #:test))
-
-#+:cormanlisp
-(defpackage "CL-PPCRE-TEST"
-  (:use "CL" "CL-PPCRE")
-  (:export "TEST"))
-
 (in-package #:cl-ppcre-test)
 
 (defparameter *cl-ppcre-test-base-directory*
@@ -67,7 +55,7 @@
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Auxiliary function used by TEST to benchmark a regex scanner
@@ -93,7 +81,7 @@ against Perl timings."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Auxiliary function used by TEST to check whether SCANNER is thread-safe."
@@ -158,7 +146,7 @@ against Perl timings."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (declare (ignorable threaded))
