@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: CL-PPCRE; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/cl-ppcre/regex-class.lisp,v 1.19 2004/11/11 21:28:00 edi Exp $
+;;; $Header: /usr/local/cvsrep/cl-ppcre/regex-class.lisp,v 1.22 2005/03/07 22:02:53 edi Exp $
 
 ;;; This file defines the REGEX class and some utility methods for
 ;;; this class. REGEX objects are used to represent the (transformed)
@@ -40,7 +40,7 @@
     (declare (optimize speed
                        (safety 0)
                        (space 0)
-                       (debug 0)
+                       (debug 1)
                        (compilation-speed 0)
                        #+:lispworks (hcl:fixnum-safety 0)))
     (defclass regex ()
@@ -256,6 +256,25 @@ defined by the user."))
          ()
       (:documentation "VOID objects represent empty regular expressions."))))
 
+(defmethod initialize-instance :after ((char-class char-class) &rest init-args)
+  (declare (optimize speed
+                     (safety 0)
+                     (space 0)
+                     (debug 1)
+                     (compilation-speed 0)
+                     #+:lispworks (hcl:fixnum-safety 0)))
+  "Make large hash tables smaller, if possible."
+  (let ((hash (getf init-args :hash)))
+    (when (and hash
+               (> *regex-char-code-limit* 256)
+               (> (hash-table-count hash)
+                  (/ *regex-char-code-limit* 2)))
+      (setf (slot-value char-class 'hash)
+              (merge-inverted-hash (make-hash-table)
+                                   hash)
+            (slot-value char-class 'invertedp)
+              (not (slot-value char-class 'invertedp))))))
+
 (declaim (ftype (function (t) simple-string) str))
 
 ;;; The following four methods allow a VOID object to behave like a
@@ -265,7 +284,7 @@ defined by the user."))
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (declare (ignore init-args))
@@ -279,7 +298,7 @@ defined by the user."))
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   0)
@@ -288,7 +307,7 @@ defined by the user."))
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "")
@@ -297,7 +316,7 @@ defined by the user."))
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   nil)
@@ -306,7 +325,7 @@ defined by the user."))
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   nil)
@@ -315,7 +334,7 @@ defined by the user."))
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (:documentation "Utility function used by the optimizer (see GATHER-STRINGS).
@@ -339,7 +358,7 @@ which are not of type STR."))
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (:documentation "Implements a deep copy of a REGEX object."))
@@ -445,7 +464,7 @@ which are not of type STR."))
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (:documentation "Returns a deep copy of a REGEX (see COPY-REGEX) and
@@ -519,7 +538,7 @@ special variable REMOVE-REGISTERS-P is true."))
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (:documentation "Returns an EVERYTHING object if REGEX is equivalent
@@ -571,7 +590,7 @@ to this object, otherwise NIL. So, \"(.){1}\" would return true
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (:documentation "Return the length of REGEX if it is fixed, NIL otherwise."))
@@ -647,7 +666,7 @@ to this object, otherwise NIL. So, \"(.){1}\" would return true
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (:documentation "Returns the minimal length of REGEX."))
@@ -703,7 +722,7 @@ to this object, otherwise NIL. So, \"(.){1}\" would return true
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   (:documentation "Returns the offset the following regex would have
