@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: CL-PPCRE; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/cl-ppcre/util.lisp,v 1.25 2004/12/18 04:14:55 edi Exp $
+;;; $Header: /usr/local/cvsrep/cl-ppcre/util.lisp,v 1.28 2005/03/07 22:02:53 edi Exp $
 
 ;;; Utility functions and constants dealing with the hash-tables
 ;;; we use to encode character classes
@@ -65,8 +65,8 @@ are discarded \(that is, the body is an implicit PROGN)."
                  bindings)
          ,@body))
 
-(defmacro rebinding (bindings &body body)
-  "REBINDING ( { var | (var prefix) }* ) form*
+(defmacro with-rebinding (bindings &body body)
+  "WITH-REBINDING ( { var | (var prefix) }* ) form*
 
 Evaluates a series of forms in the lexical environment that is
 formed by adding the binding of each VAR to a fresh, uninterned
@@ -116,7 +116,7 @@ the full Unicode support of LW, ACL, or CLISP.")
     (declare (optimize speed
                        (safety 0)
                        (space 0)
-                       (debug 0)
+                       (debug 1)
                        (compilation-speed 0)
                        #+:lispworks (hcl:fixnum-safety 0)))
     "Tests whether a character is a \"word\" character.
@@ -161,7 +161,7 @@ i.e. whether it would match [\\s] in Perl."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Returns the \"sum\" of two hashes. This is a destructive operation
@@ -183,7 +183,7 @@ on HASH1."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Returns the \"sum\" of HASH1 and the \"inverse\" of HASH2. This is
@@ -198,7 +198,7 @@ a destructive operation on HASH1."
   (declare (optimize speed
                      (safety 0)
                      (space 0)
-                     (debug 0)
+                     (debug 1)
                      (compilation-speed 0)
                      #+:lispworks (hcl:fixnum-safety 0)))
   "Tries to identify up to three intervals (with respect to CHAR<)
@@ -287,3 +287,27 @@ entries) out of the short form of VAR-LIST."
                       collect (list (first element) var))
         else
           collect (list '(function identity) element)))
+
+(defun string-list-to-simple-string (string-list)
+  (declare (optimize speed
+                     (safety 0)
+                     (space 0)
+                     (debug 1)
+                     (compilation-speed 0)
+                     #+:lispworks (hcl:fixnum-safety 0)))
+  "Concatenates a list of strings to one simple-string."
+  ;; this function provided by JP Massar; note that we can't use APPLY
+  ;; with CONCATENATE here because of CALL-ARGUMENTS-LIMIT
+  (let ((total-size 0))
+    (declare (type fixnum total-size))
+    (dolist (string string-list)
+      #-genera (declare (type string string))
+      (incf total-size (length string)))
+    (let ((result-string (make-sequence 'simple-string total-size))
+          (curr-pos 0))
+      (declare (type fixnum curr-pos))
+      (dolist (string string-list)
+        #-genera (declare (type string string))
+        (replace result-string string :start1 curr-pos)
+        (incf curr-pos (length string)))
+      result-string)))
